C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      Program cloudtop
      implicit none

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    This s the main program for the Direct Broadcast
c    version of the Cloud Top Properties Software (MODIS product MOD06).
c
c!Input Parameters:
c
c The program reads a list of input files from the cloudtop.cfg
c  file.  
c   
c  Required 
c   Inputs Are:  MODIS 1 km flat file radiances and reflectances
c                MODIS flat file geolocation parameters
c                MODIS flat file Metadata information (Day/Night
c                       flag, etc.)
c                Global Data Assimilation System (GDAS) model
c                       profiles of temperature and moisture
c                Atmospheric Coefficient Files
c                       (located in ../static directory)
c                Cloud Mask binary flat file
c 
c  Optional 
c    Inputs Are: 1 km global Olson ecosystem file
c
c!Output Parameters:
c
c  Ouput:  Writes binary flat file of float arrays of cloud top
c          properties and cloud thermodynamic phase along with
c          a 10 byte array of QA parameters.  For more specific
c          information on the arrays included in the output file,
c          see the Cloud_Top_File_Description.txt file.
c
c!References and Credits:
c   See Cloud Mask ATBD-MOD-04.
c 
c!End
c
c-----------------------------------------------------------------------

C#######################################################################
C     DECLARATIONS
C#######################################################################

c ... UW include files

      include 'db_mod06uw_data.inc'
      include 'db_mod06uw_debug.inc'
      include 'platform_name.inc'


c ... output scalars

      real success_co2, success_irp, plow, pmid, phigh,
     &     pthin, pthick, popq, pcirrus, pice, pwater,
     &     pmixed, punc, error_1km, error_geo

c ... Local arrays

      character*4 interleave_1km,interleave_geo,interl_mask
      character*80 bandnames_1km(INBAND),bandnames_geo(20),
     &             bnames_mask(NMASK),
     &             bandunits_1km(INBAND),bandunits_geo(20)
      character*132 cfgname
      
      character*(PATH_MAX) l1b_1km_hdr,l1b_geo_hdr,mask_hdr
      integer indat_bt( max_samp_line*max_samp_pixel , irphase_band+5 ),
     &        icode( max_samp_line*max_samp_pixel )
      real var ( max_samp_line*max_samp_pixel , irphase_band )
      byte qual_flag_ir ( max_samp_line*max_samp_pixel ),
     &     conf_flag_ir ( max_samp_line*max_samp_pixel )


c ... Local scalars

      real     gfit, Chisq
      integer  nscans, npixels, scan, num_arg, iargc, 
     &         npix, ng_irp, ssctpr, slcd, smcd, shcd, sthncd,
     &         sthkcd, sopcd, scicd, ni, nw, nx, nu, qual_flag,
     &         conf_flag, ci_flag, hc_flag, out_lines_5km, 
     &         out_elements_5km,maxl, clus, nmaxl, line, pixel,
     &         ncloud, nmissing, npix_scan, ngood_msk, qa_hdr_lun,
     &         beg_lin, nlins, beg_ele, neles, ibes, beg_scan,
     &         ngood_dayIRPhase, ngood_IRPhase, ngood_co2, hdr_lun,
     &         eco_1km_lun, l1b_1km_lun, geo_1km_lun, mask_lun,
     &         scan_1km_lun, anc_met_lun, qa_lun, mod06ct_lun,
     &         datatype_1km,datatype_geo,resol_mask,
     &         resolution_1km,resolution_geo,offset_1km,
     &         offset_geo,offset_mask,samples_1km,
     &         samples_geo,sampl_mask,bands_1km,
     &         bands_geo,bands_mask,lines_1km,lines_geo,
     &         lines_mask,datat_mask

c ... Save Statements
      save /platform_name_common/
     
      save / mod06_data /, / mod06_debug / 

c ... External functions

      integer copy_info
      external copy_info
      
      
C#######################################################################
C     INITIALIZATION
C#######################################################################

c ... Data initialization

      bands(1) = 20
      bands(2) = 22
      bands(3) = 23
      bands(4) = 24
      bands(5) = 25
      bands(6) = 27
      bands(7) = 28
      bands(8) = 29
      bands(9) = 30
      bands(10) = 31
      bands(11) = 32
      bands(12) = 33
      bands(13) = 34
      bands(14) = 35
      bands(15) = 36
      bands(16) = 1
      bands(17) = 6
      bands(18) = 26

c ... list bands required for IR cloud phase algorithm
      ir_mband(1) = 29
      ir_mband(2) = 31
      ir_mband(3) = 32

      ir_aband(1) = 8
      ir_aband(2) = 10
      ir_aband(3) = 11

c ... list bands needed for CO2 slicing      
      ir_co2band(1) = 36
      ir_co2band(2) = 35
      ir_co2band(3) = 34
      ir_co2band(4) = 33
      ir_co2band(5) = 31
      ir_co2band(6) = 29
      
      ir_co2bandptr(1) = 15
      ir_co2bandptr(2) = 14
      ir_co2bandptr(3) = 13
      ir_co2bandptr(4) = 12
      ir_co2bandptr(5) = 10
      ir_co2bandptr(6) = 8

c ... list bands for daytime cloud phase algorithm      
      day_band(1) = 29
      day_band(2) = 31
      day_band(3) = 32
      day_band(4) =  1
      day_band(5) =  6
      day_band(6) = 26

      day_bandptr(1) = 8
      day_bandptr(2) = 10
      day_bandptr(3) = 11
      day_bandptr(4) = 16
      day_bandptr(5) = 17
      day_bandptr(6) = 18

c ..  initialize the output metadata fields

      success_co2 = 0.0
      success_irp = 0.0
      plow    =  META_MISG
      pmid    =  META_MISG
      phigh   =  META_MISG
      pthin   =  META_MISG
      pthick  =  META_MISG
      popq    =  META_MISG
      pcirrus =  META_MISG
      pice    =  META_MISG
      pwater  =  META_MISG
      pmixed  =  META_MISG
      punc    =  META_MISG

      write( *, '(2x,''Initiated DB Cloud Top Processing  '')' ) 

c --- Get input argument of configuration file containing source
c ---  input and output file information.

c --- Check number of arguments
      num_arg = iargc()
      if (num_arg .ne. 1) then
        print *, 'Usage: cloudtop.exe cfgname'
        print *, 'where'
        print *, 'cfgname: name of cloud top configuration file'
        call exit(-1)
      endif

c ... Extract arguments
      call getarg(1, cfgname)

c ... Get debug value and processing interval information from PCF
      call db_mod06_get_rp(cfgname,debug,l1b_1km_hdr,l1b_geo_hdr,
     +                     mask_hdr,beg_lin,nlins,beg_ele,neles)

c --- Hard code Terra into platform name for now. Need new thresholds
c ---  after Aqua launch.
      platform_name = 'Terra'

c ....................................................................

c *********************************************************************

c ... Open required files for this processing section
      call db_mod06_file_open(cfgname,neles,eco_1km_lun,l1b_1km_lun,
     +                        geo_1km_lun,mask_lun,scan_1km_lun,
     +                        anc_met_lun,qa_lun,mod06ct_lun,
     +                        hdr_lun,qa_hdr_lun,h_output)

c ... Get number of scans and pixels for 1km bands

      call db_mod06_Get_Metadata(l1b_1km_hdr,l1b_geo_hdr,mask_hdr,
     &     nscans,npixels,datatype_1km,datatype_geo,datat_mask,
     &     interleave_1km,interleave_geo,interl_mask,resolution_1km,
     &     resolution_geo,resol_mask,offset_1km,offset_geo,
     &     offset_mask,samples_1km,samples_geo,sampl_mask,lines_1km,
     &     lines_geo,lines_mask,error_1km,error_geo,bands_1km,
     &     bands_geo,bands_mask,bandnames_1km,bandnames_geo,
     &     bnames_mask,bandunits_1km,bandunits_geo)


c ... Check User defined processing intervals versus scan boundaries

      call db_mod06_chk_input(nscans,npixels,beg_lin,nlins,beg_ele,
     +                        neles,beg_scan,ibes,out_lines_5km,
     +                        out_elements_5km)

C#######################################################################
C     INITIALIZATION OF GRANULE BASED VARIABLES
C#######################################################################

      npix = 0

c ... Initialize metadata counters for both products

      call db_init_granule_var_mod06ir( ni, nw, nx, nu, ng_irp )

      call db_init_granule_var_mod06ct( ssctpr, slcd, smcd,
     +                   shcd, sthncd, sthkcd, sopcd, scicd )

C#######################################################################
C     START SCIENCE PROCESSING
C#######################################################################

c ... Begin scan loop

      do scan = beg_scan , nscans

c ...   Write debug info

        write( *, '(''Processing scan # '',i4)' ) scan

c ...   Extract radiance, geolocation, mask, and ancillary data

        call db_mod06_get_data(l1b_1km_lun,geo_1km_lun,datatype_1km,
     &       datatype_geo,datat_mask,interleave_1km,interleave_geo,
     &       interl_mask,resolution_1km,resolution_geo,resol_mask,
     &       offset_1km,offset_geo,offset_mask,samples_1km,
     &       samples_geo,sampl_mask,lines_1km,lines_geo,lines_mask,
     &       error_1km,error_geo,bands_1km,bands_geo,bands_mask,
     &       bandnames_1km,bandnames_geo,bnames_mask,bandunits_1km,
     &       bandunits_geo,scan_1km_lun,anc_met_lun,mask_lun,
     &       eco_1km_lun,scan)


c ...   Initialize output product arrays for this scan

        call db_mod06CT_initialize_output()

        call db_mod06IR_initialize_output( qual_flag_ir, conf_flag_ir,
     &    maxl, clus, gfit, Chisq, nmaxl, icode, indat_bt, var)

        npix_scan = 0

c ...   Start loop over pixel boxes (line and pixel is at corner of box)

        do line = 1 , ( max_line / isamp ) * isamp, isamp

          do pixel = ibes , ( npixels / isamp ) * isamp, isamp

             npix = npix + 1
             npix_scan = npix_scan + 1

c ...        Initialize scan based qa variables

             call db_init_scan_var_mod06ct( qual_flag, conf_flag,
     &         ci_flag, hc_flag)

c ...        Check this box for good data

             call db_mod06_check_box( line, pixel, nmissing, ncloud,
     &         ngood_msk, ngood_co2, ngood_IRphase, ngood_dayIRPhase )

c ...        If this box has enough good pixels, compute products
c ...        First CO2 slicing

             if( ngood_co2 .ge. min_ngoodct ) then
               call db_mod06CT_compute_products(cfgname,line,pixel,
     &          ngood_msk,ncloud, qual_flag, conf_flag, hc_flag, 
     &          ci_flag, ssctpr, slcd, smcd, shcd, sthncd, sthkcd, 
     &          sopcd, scicd )
             end if

c ...        For IR phase, fill processing variables

             if ( ngood_IRphase .ge. min_ngood )
     &         call db_put_proc_var_irphase( npix_scan, line, pixel,
     &           ncloud, nmissing,ngood_msk,indat_bt,var)

c ...        Set the QA information for this box

             call db_mod06ct_set_qa( line, pixel, nmissing,
     &         ncloud, qual_flag, conf_flag, hc_flag, ci_flag )

c ...        Fill up the common output arrays

             call db_fill_common_arrays(line,pixel)

          end do

        end do

c ...   Compute the cloud phase product for this scan

        call db_get_irphase( npix_scan, indat_bt, icode)

c ...   Set QA information for cloud phase and sum metadada counters

        call db_mod06ir_set_qa( npixels,qual_flag_ir, conf_flag_ir,
     &    maxl, clus, gfit, Chisq, nmaxl, icode, ng_irp,
     &    ni, nw, nx, nu )

c ...   Place phase processing arrays in correct output formatted arrays

        call db_put_irphase_out_arrays(npixels, icode, indat_bt, var)

c ...   Write products for this scan

c ...   write data to output file 
        call db_mod06_write_products(mod06ct_lun,hdr_lun,qa_lun,
     &       qa_hdr_lun,scan,nscans,out_elements_5km)

c ... End of loop over all scans

      end do

C#######################################################################
C     END SCIENCE PROCESSING
C#######################################################################

c ... Get metadata for MOD06

      call db_mod06uw_stats_out( ssctpr, ng_irp,
     &  npix, slcd, smcd, shcd, sthncd, sthkcd,
     &  sopcd, scicd, ni, nw, nx, nu, success_co2,
     &  success_irp, plow, pmid, phigh, pthin,
     &  pthick, popq, pcirrus, pice, pwater,
     &  pmixed, punc)

c ... close files for this section of code

      call db_mod06_file_close(l1b_1km_lun,eco_1km_lun,geo_1km_lun,
     &                         mask_lun,scan_1km_lun,anc_met_lun,
     &                         mod06ct_lun,hdr_lun,qa_lun,qa_hdr_lun)

c ... Write debug info and then close debug file

      write( *, '(2x,''DB Cloud Top Processing Completed. '')' ) 

      close( h_output )

      end
