C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_get_trans(cfgname,t,w,ozone,theta,band_index,tau)

      implicit none
      save

c-----------------------------------------------------------------------
c!F77
c
c!Description:
c    Convert input profiles from 40 to 42 levels.
c    Call Terra/Aqua MODIS 42-level fast transmittance routine.
c    Convert output transmittance profiles from 42 to 40 levels.
c
c!Input Parameters:
c    cfgname     Configuratin file name
c    t           Temperatures at 40 standard levels (Kelvin)
c    w           Water vapor mixing ratios at 40 standard levels (g/kg)
c    ozone       Ozone concentrations at 40 standard levels (ppmv)
c    theta       Local zenith angle from surface to satellite (degrees)
c    band_index  Index pointing to MODIS IR band number 
c
c!Output Parameters:
c    tau      Total transmittances (no units) at 40 standard levels
c
c!Revision History:
c
c!Team-unique Header:
c
c!End
c-----------------------------------------------------------------------

      include 'db_mod06uw_data.inc'
      include 'db_mod06uw_debug.inc'

c     Scalar arguments.
      real theta
      integer band_index

c     Array arguments.
      character*(*) cfgname
      real t(plev), w(plev), ozone(plev), tau(plev,nbct)

c     Local scalars.
      integer kban,jdet,ii

c     Local arrays.
      real temp(tau_lev), wvmr(tau_lev), ozmr(tau_lev), taut(tau_lev)
      integer mbnds(nbct)

c     External subroutines
      external db_tran_modisd042

c-----------------------------------------------------------------------

c     Data Statements.

c     Order of MODIS IR band numbers used
      data mbnds/36,35,34,33,31/

c     Dectector number.
      data jdet /0/

c-----------------------------------------------------------------------

      if(band_index .eq. 1) then

c       Copy 40 level atmospheric profile into 42 level arrays.
        do ii = 1, tau_lev
          temp(ii) = t(min(ii, plev))
          wvmr(ii) = w(min(ii, plev))
          ozmr(ii) = ozone(min(ii, plev))
        end do

      end if

      kban = mbnds(band_index)
      call db_tran_modisd042(cfgname,temp,wvmr,ozmr,theta,kban,
     &                       jdet,taut)

c     Extract first 40 levels from 42 level transmittance profile
      do ii = 1, plev
        tau(ii, band_index) = taut(ii)
      end do

      return
      end
