C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_mod06_get_data(l1b_1km_lun,geo_1km_lun,datatype_1km,
     &           datatype_geo,datat_mask,interleave_1km,interleave_geo,
     &           interl_mask,resolution_1km,resolution_geo,resol_mask,
     &           offset_1km,offset_geo,offset_mask,samples_1km,
     &           samples_geo,sampl_mask,lines_1km,lines_geo,lines_mask,
     &           error_1km,error_geo,bands_1km,bands_geo,bands_mask,
     &           bandnames_1km,bandnames_geo,bnames_mask,bandunits_1km,
     &           bandunits_geo,scan_1km_lun,anc_met_lun,mask_lun,
     &           eco_1km_lun,scan)

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    Get all input data at 1km resolution needed by the University
c    of Wisconsin MOD06 Direct Broadcast product.
c
c!Input Parameters:
C    l1b_1km_lun     LUN for open 1km L1B data file
C    geo_1km_lun     LUN for open geolocation 1km file
C    datatype_1km    Format of data in L1b file
C    datatype_geo    Format of data in geolocation file
C    datat_mask      Format of data in cloud mask file
C    interleave_1km  Order data is saved in L1b file
C    interleave_geo  Order data is saved in geolocation file
C    interl_mask     Order data is saved in cloud mask file
C    resolution_1km  L1b file data resolution
C    resolution_geo  Geolocation file data resolution
C    resol_mask      Cloud mask file data resolution
C    offset_1km      Offset, if any, of 1km L1B data
C    offset_geo      Offset, if any, of geolocation file data
C    offset_mask     Offset, if any, of cloud mask file data
C    samples_1km     Number of elements in 1km L1b file
C    samples_geo     Number of elements in geolocation file
C    sampl_mask      Number of elements in cloud mask file
C    lines_1km       Number of lines of data in 1km L1b file
C    lines_geo       Number of lines of data in geolocation file
C    lines_mask      Number of lines of data in cloud mask file
C    error_1km       Bad data value of data in 1km L1b file
C    error_geo       Bad data value of data in geolocation file
C    bands_1km       Number of bands in 1km L1b file
C    bands_geo       Number of bands in geolocation file
C    bands_mask      Number of bands in cloud mask file
C    bandnames_1km   Band numbers in 1km L1b file
C    bandnames_geo   Band numbers in geolocation file
C    bnames_mask     Band numbers in cloud mask file
C    bandunits_1km   Units for each band in the 1km L1b data file
C    bandunits_geo   Units for each band in the geolocation file
C    scan_1km_lun    LUN for scan based metadata file
C    anc_met_lun     LUN for open met data file
C    mask_lun        LUN for open Cloud Mask file
C    eco_1km_lun     LUN for open 1km ecosystem file
c    scan            Scan number within L1B granule
c
c!Output Parameters:
c    The following arrays in COMMON /MOD06_DATA/ are filled:
c    RADIANCE1      Radiances for IR bands
c    LAT1           Latitude
c    LON1           Longitude
c    TEM1           Surface temperature
c    WAT1           Surface water vapor mixing ratio
c    OZN1           Total column ozone
c    PRE1           Surface pressure
c    TPROF1         Temperatures at 40 standard levels (see array PSTD)
c    WPROF1         Water vapor mixing ratio at 40 standard levels
c    LAND1          Land/water mask
c    ECO            Ecosystem value based on Olson categorization
c    VIEW           Geolocation viewing zenith angle value
c    SOLZ           Geolocation solar zenith angle value
c    DN_FLAG        Pixel Day/night flag taken from instrument mode
c                    and solar zenith angle threshold
c    MASK1          Cloudmask
c    CS_BIAS_CORR   Clear-sky radiance bias correction for CO2-slicing
c
c!Revision History:
c
c!Team-unique Header:
c
c!End
c
c-----------------------------------------------------------------------

      implicit none
      
      include 'db_mod06uw_data.inc'
      include 'db_mod06uw_debug.inc'

c ... arguments

      integer eco_1km_lun,l1b_1km_lun,geo_1km_lun,scan_1km_lun,
     +        mask_lun,scan,datat_mask,datatype_1km,datatype_geo,
     +        resolution_1km,resolution_geo,resol_mask,offset_1km,
     +        offset_geo,offset_mask,samples_1km,samples_geo,sampl_mask,
     +        lines_1km,lines_geo,lines_mask,bands_1km,bands_geo,
     +        bands_mask,anc_met_lun

      real error_1km,error_geo,error_mask
 
      character*(*) interleave_1km,bandnames_1km(*),bandunits_1km(*),
     +              interleave_geo,bandnames_geo(*),bandunits_geo(*),
     +              interl_mask,bnames_mask(*)

c ... local variables

      integer nise, resol, data_size( 2 )
      character*1 scan_type
      character*80 bunits_mask(NMASK)
      character*6 cal_type,band_no
      character*80 req_bandname
      
      real buffer( max_pixel *  max_line )
      byte v_flag( max_pixel *  max_line ), 
     &        m_buf (max_pixel *  max_line)

      integer i, j, k, m, n, mask_byte, mirror_side, 
     &        scan_flag, status, lun_ozone, lun_icec,
     &        lun_sst, lun_nisen, lun_nises

      real pres( 16 ), temp( 16 ), mixr( 16 ), land, sfctmp, prmsl,
     &  pwat, ugrd, vgrd, ozone, icec, sst

      integer ni
      real pstd( 40 ), tstd( 40 ), wstd( 40 ), 
     &  latitude
      
      integer imet, jmet, last_imet, last_jmet
      real x, x0, dx, y, y0, dy

c ... set program name for error messaging
      character*32 routine_name
      parameter ( routine_name = 'db_mod06_get_data' )

c ... set small value used to discriminate water vapor values
      real       EPS_ZERO
      parameter (EPS_ZERO = 0.00001)

c ... external functions

      integer db_get_swath_metadata,db_read_flat_file,db_get_ancillary,
     &        db_read_mask_file
      external db_get_swath_metadata,db_read_flat_file,db_get_ancillary,
     &        db_read_mask_file

c ... Save Statements
      SAVE / mod06_data /, / mod06_debug /

c ... Data Statements

c ... Define 40 standard pressure levels (hPa)

      data pstd / .1,.2,.5,1.,1.5,2.,3.,4.,5.,7.,10.,15.,20.,25.,30.,
     & 50.,60.,70.,85.,100.,115.,135.,150.,200.,250.,300.,350.,400.,
     & 430.,475.,500.,570.,620.,670.,700.,780.,850.,920.,950.,1000. /


c ... Initialize 1km radiance storage array with the bad value

      do k = 1, nbands
        do j = 1, max_line
          do i = 1, max_pixel
            radiance1( i, j, k ) = bad_value
          end do
        end do
      end do

c ... Extract the scan type indicating day or night mode from L1B file

      scan_flag = -1
      mirror_side = -1
      status = db_get_swath_metadata(scan_1km_lun,scan,scan_flag,
     &                               mirror_side)
      if (status .ne. 0) then
          call message(routine_name,
     &                'FAILED - Unable to extract scan metadata. ',
     &                 0, 2)
      endif

c ... Convert Scan Flag into something that get_L1b can understand
      scan_type = ' '
      if(scan_flag .eq. 1) then
         scan_type = 'D'
      endif

      do k = 1, nbands

c ...   Initialize buffer arrays

        m = 1 
        do j = 1, max_line
          do i = 1, max_pixel
            buffer( m ) = bad_value
            v_flag( m ) = -1
            m = m + 1
          end do
        end do

c ...   Get single band of radiance data

        resol = 1
c ...   Get data in terms of reflectance if visible channel, and
c ..     radiance if a thermal channel.
        if (bands(k).le.19 .or. bands(k).eq.26) then
            cal_type = 'ref'
        else
            cal_type = 'rad'
        endif

        write(req_bandname,*) bands(k)

        status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                             l1b_1km_lun,datatype_1km,
     &                             interleave_1km,resolution_1km,
     &                             error_1km,offset_1km,samples_1km,
     &                             lines_1km,bands_1km,bandnames_1km,
     &                             bandunits_1km,max_pixel,max_line,
     &                             buffer,v_flag,data_size(1),
     &                             data_size(2))
        if (status .lt. 0) then
          if ( debug .gt. 0 ) then
            write(h_output, '(''Error reading L1B radiance'')' )
            write(h_output, '(5x,''read status = '',I9)') status
          endif
          write(band_no,'(I6)' ) bands(k)
          call message( routine_name,
     &      'Failed to extract 1km L1B data. for band ' //band_no
     &      ,0, 2 )
        endif

c ...   Copy good data into 1km radiance storage array

        n = 1
        do j = 1, data_size( 2 )
          do i = 1, data_size( 1 )
            if ( v_flag( n ) .eq. 0 .and. buffer( n ) .gt. 0.0 )
     &        radiance1( i, j, k ) = buffer( n )
            n = n + 1
          end do
        end do

      end do

c ... Get geolocation data from file

c ... Extract latitude information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          lat1( i, j ) = bad_value
          buffer( m ) = bad_value
          v_flag( m ) = -1
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'Latitude'
      cal_type = 'deg'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Latitude. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Latitude information ' 
     &     ,0, 2 )
      endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          if ( buffer( n ) .ge. -90.0 .and.
     &      buffer( n ) .le. 90.0 ) lat1( i, j ) = buffer( n )
            n = n + 1
        end do
      end do

c ... Extract longitude information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          lon1( i, j ) = bad_value
          buffer( m ) = bad_value
          v_flag( m ) = -1
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'Longitude'
      cal_type = 'deg'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Longitude. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Longitude information '
     &     ,0, 2 )
       endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          if ( buffer( n ) .ge. -180.0 .and.
     &      buffer( n ) .le. 180.0 ) lon1( i, j ) = buffer( n )
            n = n + 1
        end do
      end do


c ... Extract viewing zenith information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          view( i, j ) = bad_value
          buffer( m ) = bad_value
          v_flag( m ) = -1
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'SensorZenith'
      cal_type = 'deg'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Viewing Zenith. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Viewing Zenith information ' 
     &     ,0, 2 )
       endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          if (abs(buffer(n)) .lt. 67.0 .and. abs(buffer(n)).gt. 0.0)
     &      view( i, j ) = buffer( n )
            n = n + 1
        end do
      end do


c ... Extract solar zenith information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          solz( i, j ) = bad_value
          dn_flag( i, j  ) = -1
          buffer( m ) = bad_value
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'SolarZenith'
      cal_type = 'deg'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Solar Zenith. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Solar Zenith information ' 
     &     ,0, 2 )
       endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
            solz( i, j ) = buffer( n )
            if (solz(i,j) .gt. 0.0) then
               if (scan_type .ne. 'D' .or. solz(i,j) .gt. 85.0) then
                 dn_flag(i,j) = 0
               else
                 dn_flag(i,j) = 1
               endif
            endif
            n = n + 1
        end do
      end do


c ... Extract land/sea mask information
 
      m = 1
      do j = 1, max_line
        do i = 1, max_pixel
          land1( i, j ) = -1
          eco( i, j ) = -1
          buffer( m ) = bad_value
          m = m + 1
        end do
      end do
 
      resol = 1
      req_bandname = 'LandSea'
      cal_type = 'flg'

      status = db_read_flat_file(scan,resol,req_bandname,cal_type,
     &                           geo_1km_lun,datatype_geo,
     &                           interleave_geo,resolution_geo,
     &                           error_geo,offset_geo,samples_geo,
     &                           lines_geo,bands_geo,bandnames_geo,
     &                           bandunits_geo,max_pixel,max_line,
     &                           buffer,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Land Sea Mask. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Land Sea Mask information ' 
     &     ,0, 2 )
       endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          if ( buffer( n ) .ge. 0.0 .and.
     &      buffer( n ) .le. 10.0 ) land1( i, j ) =
     &      int( buffer( n ) )
            n = n + 1
        end do
      end do

c-----------------------------------------------------------------------
c     READ NON-MODIS ANCILLARY DATA
c-----------------------------------------------------------------------

      if (eco_1km_lun .gt. 0) then
          call read_goode( data_size( 1 ), data_size( 2 ),
     &    lat1, lon1, eco_1km_lun, eco )
      endif

c ... Get ancillary data for 1km bands

      last_imet = -1
      last_jmet = -1
      lun_icec = 0
      lun_ozone = 0
      lun_sst = 0
      lun_nisen = 0
      lun_nises = 0

      do j = 1, max_line
        do i = 1, max_pixel

c ...     Set default output values

          tem1( i, j ) = bad_value
          wat1( i, j ) = bad_value
          ozn1( i, j ) = bad_value
          pre1( i, j ) = bad_value
          do k = 1, 40
            tprof1( k, i, j ) = bad_value
            wprof1( k, i, j ) = bad_value
          end do

c *******************************************************************
c ...     Get clear-sky radiance biases here, either in call to
c ...     get_ancillary or in a separate call. Will return a bias
c ...     value (calculated-observed) for five IR channels (#s 31,
c ...     33, 34, 35, 36). For now, fill the array with "bad_values".
c ...     One value applies to all pixels in the max_line X max_pixel
c ...     carray.
          do k = 1,nbct
            cs_bias_corr(k,i,j) = bad_value
          end do
c *******************************************************************

c ...     Get the ancillary data necessary to compute products
c ...      If lat/lon data is ok, get ancillary data

          if ( lat1( i, j ) .ge. -90.0 .and.
     &      lat1( i, j ) .le.  90.0 .and.
     &      lon1( i, j ) .ge. -180.0 .and.
     &      lon1( i, j ) .le.  180.0 ) then

            do k = 1, 16
              pres( k ) = bad_value
              temp( k ) = bad_value
              mixr( k ) = bad_value
            end do
            land = bad_value
            sfctmp = bad_value
            prmsl = bad_value
            ozone = bad_value

            status=db_get_ancillary(anc_met_lun,lun_ozone,lun_icec,
     &        lun_sst,lun_nisen,lun_nises,lat1(i,j),lon1(i,j),
     &        pres,temp,mixr,land,sfctmp,prmsl,pwat,ugrd,vgrd,
     &        ozone,icec,sst,nise) 
            if (status .lt. 0) then 
                call message( 'routine_name',
     &                        'Error reading ancillary files.',
     &                         0, 2 )
            endif


c ...       Set surface data items if values are ok

            if ( sfctmp .ge. 0.0 ) tem1( i, j ) = sfctmp
            if ( mixr( 1 ) .ge. 0.0 ) wat1( i, j ) = mixr( 1 )
            if ( ozone .ge. 0.0 ) ozn1( i, j ) = ozone
            if ( prmsl .ge. 0.0 ) pre1( i, j ) = prmsl

c ...       Compute cell coordinates in met grid

            x = min( max( lon1( i, j ),  -179.99 ), 179.99 )
            if( x .lt. 0.0 ) x = lon1( i, j ) + 360.0
            x0 = 0.0
            dx = 1.0
            imet = int( ( x - x0 + 0.5*dx ) / dx )
            if( imet .eq. 360 ) imet = 0

            y = min( max( lat1( i, j ), -89.99 ), 89.99 )
            y0 = 90.0
            dy = -1.0
            jmet = int( ( y - y0 + 0.5*dy ) / dy )

c ...       If this is a new grid cell, interpolate temperature and
c ...       water vapor mixing ratio profiles to the 40 standard levels

            if ( imet .ne. last_imet .or. jmet .ne. last_jmet ) then
              last_imet = imet
              last_jmet = jmet
              ni = 16
              latitude = lat1( i, j )
              call extend_profile( ni, pres, temp, mixr, latitude,
     &          tstd, wstd )
            endif

c ...       Copy temperature and moisture profiles into output array

            do k = 1, 40
              if ( tstd( k ) .ge. 0.0 ) tprof1( k, i, j ) = tstd( k )

c ...         Allow very small values of water vapor
              if ( abs( wstd( k ) ) .lt. EPS_ZERO) then
                  wprof1( k, i, j ) = EPS_ZERO
              elseif ( wstd( k ) .ge. EPS_ZERO ) then
                  wprof1( k, i, j ) = wstd( k )
              endif
            end do


c ...       If this is a water pixel, then make sure ecosystem is
c ...       is not saying land - set equal to 14
            if (land1(i,j).eq.0.or.land1(i,j).eq.3.or.land1(i,j).eq.5
     +        .or. land1(i,j).eq.6 .or. land1(i,j).eq.7)
     +        eco(i,j) = 14

          endif

        end do
      end do

c ... Get cloud mask result array 

      m = 1
      do k = 1, max_line
        do j = 1, max_pixel
          mask1( j, k ) = 0
          m_buf( m ) = 0
          m = m + 1
        end do
      end do

      req_bandname = 'byte 1'
      resol = 1
      cal_type = ' '
      error_mask = 0.0
      do n = 1 , NMASK
         bunits_mask(n) = '  '
      enddo
      status = db_read_mask_file(scan,resol,req_bandname,cal_type,
     &                           mask_lun,datat_mask,interl_mask,
     &                           resol_mask,error_mask,
     &                           offset_mask,sampl_mask,
     &                           lines_mask,bands_mask,bnames_mask,
     &                           bunits_mask,max_pixel,max_line,
     &                           m_buf,v_flag,data_size(1),
     &                           data_size(2))
      if (status .lt. 0) then
        if ( debug .gt. 0 ) then
          write(h_output, '(''Error reading Cloud Mask Byte 1. '')' )
          write(h_output, '(5x,''read status = '',I9)') status
        endif
        call message( routine_name,
     &    'Failed to extract Cloud Mask information '
     &     ,0, 2 )
       endif

c ... Fill array
      n = 1
      do j = 1, data_size( 2 )
        do i = 1, data_size( 1 )
          mask1( i, j ) = m_buf( n )
          n = n + 1
        end do
      end do

c ...............................................................
c ... Write debug information ...................................

      if ( debug .gt. 0 ) then


        write( h_output, '(/,a,/)' ) 'mod06_get_data debug'

        i = 676
        j = 6

        write( h_output, '( ''Data for Nadir Pixel at '', ' //
     &    ' ''Scan'',i4,'', Pixel'',i4,'', Line'',i2)' ) scan, i, j

        write( h_output, '(''IR Bands: '',20i7)' )
     &    ( bands( k ), k = 1, nbands )
        write( h_output, '(''Radiance: '',20f7.3)' )
     &    ( radiance1( i, j, k ), k = 1, nbands )

        mask_byte = int( mask1( i, j ) )
        write( h_output,
     &    ' (''Lat ='',f9.3,'', Lon ='',f10.3,'', Landmask ='',i3, ' //
     &    ' '', Cloudmask Bit 0 ='',i2,'', Cloudmask Bits 1:2 ='',i3)')
     &    lat1( i, j ), lon1( i, j ), land1( i, j ),
     &    iand( mask_byte, 1 ),
     &    iand( mask_byte, 6 )

        write( h_output, '(''Ancillary Data: '', ' //
     &    ' ''Sfc Temp ='',f7.2,'' K, Mixing Ratio ='',f7.2, ' //
     &    ' '' g/kg, Total Ozone ='',f7.2,'' Dobsons,'', ' //
     &    ' '' Sfc Pressure ='',f7.2,'' hPa'')' )
     &    tem1( i, j ), wat1( i, j ), ozn1( i, j ), pre1( i, j )

        write( h_output, '(''Auxillary Data: '', ' //
     &    ' ''Viewing Zenith ='',f7.2,'' Ecosystem Type ='',i10 )')
     &    view( i, j ), eco( i, j )

        write( h_output, '(''Auxillary Data: '', ' //
     &    ' ''Solar Zenith ='',f7.2,'' Day/Night Flag  ='',i10 )')
     &    solz( i, j ), dn_flag( i, j )

         write( h_output, '(''Pres(hPa)  Temp(K)  Mixr(g/kg)'')' )
         do k = 1, 40
            write( h_output, '(f9.1,2x,f7.2,2x,f10.3)' )
     &      pstd(k), tprof1(k,i,j), wprof1(k,i,j)
         end do
c ...............................................................

      endif




      end
