C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_mod06_write_products(out_handle,hdr_lun,qa_handle,
     &                                   qa_hdr_lun,scan,nscans,
     &                                   nboxes )

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    Write MOD06 retrieval products and QA data to the output file.
c
c!Input Parameters:
c    OUT_HANDLE    LUN for MOD06CT output file
c    HDR_LUN       LUN for product header output file
c    QA_HANDLE     LUN for MOD06CT QA product
c    QA_HDR_LUN    LUN for QA product header output file
c    SCAN          Scan number within L1B granule
c    NSCANS        Total number of scans in this data set
c    NBOXES        Number of retrieval boxes along the scan
c                  (e.g. 1354 pixels, 5x5 sampling gives 1354/5 boxes)
c
c!Output Parameters:
c    None
c
c    The following arrays in COMMON /MOD06_DATA/ are written to output:
c    BRIGHTNESS_TEMP       Brightness temperature
c    SFC_TEMP              Surface temperature
c    SFC_PRES              Surface pressure
c    PRODUCT_QA            Product run time QA
c
c!Revision History:
c
c!Team-unique Header:
c
c!End
c
c-----------------------------------------------------------------------
      
      implicit none


      include 'db_mod06uw_data.inc'
      
c ... arguments

      integer out_handle, scan, nscans, nboxes, hdr_lun, qa_handle,
     &        qa_hdr_lun

c ... local variables

      integer i , j, ii

      save / mod06_data / 

c ... local arrays
      character*2 band_no
      character*3 b_unit
      character*5 ratios(MAX_SOLUTNS)
      character*7 btdif(PHASE_PAIRS)
      integer m_bands(n_output),f_bands(max_solutns)
      real ctp_night(max_samp_pixel,max_samp_line),
     +     ctp_day(max_samp_pixel,max_samp_line),
     +     ctt_night(max_samp_pixel,max_samp_line),
     +     ctt_day(max_samp_pixel,max_samp_line),
     +     cee_night(max_samp_pixel,max_samp_line),
     +     cee_day(max_samp_pixel,max_samp_line),
     +     cld_irp_night(max_samp_pixel,max_samp_line),
     +     cld_irp_day(max_samp_pixel,max_samp_line),
     +     fraction_night(max_samp_pixel,max_samp_line),
     +     fraction_day(max_samp_pixel,max_samp_line)

c ... data statments
      data m_bands / 29, 31, 32, 33, 34, 35, 36 /
      data f_bands / 36, 35, 34, 33, 31/
      data ratios / '36/35', '35/34', '35/33', '34/33', '33/31' /
      data btdif / 'B29-B31', 'B31-B32'/

c ... Initialize day/night arrays
      do j = 1 , max_samp_line
         do i = 1 , max_samp_pixel
            ctp_night(i,j) = bad_value
            ctt_night(i,j) = bad_value
            cee_night(i,j) = bad_value
            cld_irp_night(i,j) = bad_value
            fraction_night(i,j) = bad_value
            ctp_day(i,j) = bad_value
            ctt_day(i,j) = bad_value 
            cee_day(i,j) =  bad_value
            cld_irp_day(i,j) = bad_value
            fraction_day(i,j) = bad_value
         enddo
      enddo

c ..  Separate 4 arrays into day and night products

      do j = 1 , max_samp_line
         do i = 1 , max_samp_pixel
            if (day_night_flag(i,j) .eq. 0) then
                ctp_night(i,j) = cloudtop_pres(i,j)
                ctt_night(i,j) = cloudtop_temp(i,j)
                cee_night(i,j) = cloudtop_eff_emi(i,j)
                cld_irp_night(i,j) = cloud_phase(i,j)
                fraction_night(i,j) = cloud_fraction(i,j)
            elseif (day_night_flag(i,j) .eq. 1) then
                ctp_day(i,j) = cloudtop_pres(i,j)
                ctt_day(i,j) = cloudtop_temp(i,j)
                cee_day(i,j) = cloudtop_eff_emi(i,j)
                cld_irp_day(i,j) = cloud_phase(i,j)
                fraction_day(i,j) = cloud_fraction(i,j)
            endif
         enddo
      enddo
c ... Write each product array for this scan to output file

      b_unit = 'tmp'
      do i = 1 , n_output
        write(band_no,'(I2)') m_bands(i)
        call write_output( out_handle, hdr_lun, scan, nscans, nboxes, i,
     &    brightness_temp(1,1,i), 'Brightness_Temperature_B'// band_no ,
     &    b_unit )
      end do

      ii = 1 + n_output
      b_unit = 'tmp'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  sfc_temp, 'Surface_Temperature' , b_unit )

      ii = ii + 1
      b_unit = 'msl'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  sfc_pres, 'Surface_Pressure' , b_unit )

      ii = ii + 1
      b_unit = 'flg'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  processing_flag, 'Processing_Flag' , b_unit )

      ii = ii + 1
      b_unit = 'flg'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cloud_h_method, 'Cloud_Height_Method' , b_unit )

      ii = ii + 1
      b_unit = 'hPa'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cloudtop_pres, 'Cloud_Top_Pressure' , b_unit )

      ii = ii + 1
      b_unit = 'hPa'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  ctp_night, 'Cloud_Top_Pressure_Night' , b_unit )

      ii = ii + 1
      b_unit = 'hPa'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  ctp_day, 'Cloud_Top_Pressure_Day' , b_unit )

      ii = ii + 1
      b_unit = 'tmp'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cloudtop_temp, 'Cloud_Top_Temperature' , b_unit )

      ii = ii + 1
      b_unit = 'tmp'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  ctt_night, 'Cloud_Top_Temperature_Night' , b_unit )

      ii = ii + 1
      b_unit = 'tmp'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  ctt_day, 'Cloud_Top_Temperature_Day' , b_unit )

      ii = ii + 1
      b_unit = 'hPa'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  height_tropopause, 'Tropopause_Height' , b_unit )

      ii = ii + 1
      b_unit = 'pct'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cloud_fraction, 'Cloud_Fraction' , b_unit )

      ii = ii + 1
      b_unit = 'pct'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  fraction_night, 'Cloud_Fraction_Night' , b_unit )

      ii = ii + 1
      b_unit = 'pct'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  fraction_day, 'Cloud_Fraction_Day' , b_unit )

      ii = ii + 1
      b_unit = 'pct'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cloudtop_eff_emi, 'Cloud_Effective_Emissivity' , b_unit )

      ii = ii + 1
      b_unit = 'pct'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cee_night, 'Cloud_Effective_Emissivity_Night' , b_unit )

      ii = ii + 1
      b_unit = 'pct'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cee_day, 'Cloud_Effective_Emissivity_Day' , b_unit )

      ii = ii + 1
      b_unit = 'hPa'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cloudtop_pre_ir, 'Cloud_Top_Pressure_Infrared' , b_unit )

      b_unit = 'rad'
      do i = 1, max_solutns
        ii = ii + 1
        write(band_no,'(I2)') f_bands(i)
        call write_output(out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &    spec_cloud_forcing(1,1,i), 'Spectral_Cloud_Forcing_B' 
     &    // band_no , b_unit )
      end do
            
      b_unit = 'hPa'
      do i = 1, max_solutns
        ii = ii + 1
        call write_output(out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &    cloudtop_pres_from_ratios(1,1,i), 
     &    'Cloud_Top_Pressure_From_Ratios_' // ratios(i) , b_unit )
      end do

      ii = ii + 1
      b_unit = 'flg'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  surface_type, 'Surface_Type' , b_unit )

      b_unit = 'rad'
      do i = 1, n_output
        ii = ii + 1
        write(band_no,'(I2)') m_bands(i)
        call write_output(out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &    radiance_variance(1,1,i), 'Radiance_Variance_B' // band_no ,
     &    b_unit )
      end do      

      b_unit = 'tmp'
      do i = 1, phase_pairs
        ii = ii + 1
        call write_output(out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &    brightness_diff(1,1,i), 'Brightness_Temperature_Difference_' 
     &    // btdif(i) , b_unit )
      end do      

      ii = ii + 1
      b_unit = 'flg'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes,  ii,
     &  cloud_phase, 'Cloud_Phase_Infrared', b_unit )

      ii = ii + 1
      b_unit = 'flg'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cld_irp_night, 'Cloud_Phase_Infrared_Night' , b_unit )

      ii = ii + 1
      b_unit = 'flg'
      call write_output( out_handle, hdr_lun, scan, nscans, nboxes, ii,
     &  cld_irp_day, 'Cloud_Phase_Infrared_Day' , b_unit )

c ... Write all QA arrays for this scan to output file

      call write_qa(qa_handle, qa_hdr_lun, scan, nscans, nboxes )
                  
      end

c-----------------------------------------------------------------------

      subroutine write_output(out_handle,hdr_lun,scan,nscans,nboxes,
     &                        req_band,array,arrnm,b_unit_name)
c-----------------------------------------------------------------------
c!F77
c
c!Description:
c    Write MOD06 retrieval products to the output file.
c
c!Input Parameters:
c    OUT_HANDLE    LUN for MOD06CT output file
c    HDR_LUN       LUN for product header output file
c    SCAN          MODIS scan number
c    NSCANS        Total number of scans in this data set
c    NBOXES        Number of 5x5 retrieval boxes for this scan
c    REQ_BAND      Output file band number
c    ARRAY         Array of product values
c                  (Bad values should be set to BAD_VALUE in the calling
c                  routine: see mod06uw_data.inc for BAD_VALUE value)
c    ARRNM         Name of the array (SDS name) in the output file
c
c!Output Parameters:
c    None
c
c!Revision History:
c    12-DEC-1997 Kathy Strabala 
c                Changed BYTE to INTEGER*1 so that mapi would 
c                like it
c    10-DEC-1997 Liam Gumley
c                Added BYTE data types
c                Now gets _FillValue directly from output file
c
c!Team-unique Header:
c
c!End
c-----------------------------------------------------------------------

      implicit none
      save

      
      include 'db_mod06uw_data.inc'
      include 'db_mod06uw_debug.inc'
      
c ... arguments

      integer out_handle, scan, nboxes, nscans, req_band, hdr_lun
      real array(max_samp_pixel,max_samp_line)
      character*(*) arrnm, b_unit_name
      logical init

c ... local variables
            
      integer i, j, out, ii, out_samples, out_lines, rtn, fil_datatype,
     &        fil_lun, fil_resolution, fil_offset, fil_samples, 
     &        fil_lines, fil_bands, out_maxsamples, out_maxlines,
     &        req_lines, req_samples, hdr_flg
      

      real fill_float, fil_error

c ... Local arrays
      byte out_flag(max_samp_pixel*max_samp_line)
      character*4 fil_interleave
      character*72 text
      character*3 fil_bandunits(OUT_BAND)
      character*80 fil_bandnames(OUT_BAND)
      character*256 fil_desc
      real float_data( max_samp_pixel * max_samp_line )

c ... external routines
      external db_write_flat_file
      integer db_write_flat_file

      data init /.true./, ii /0/

 
      if (init) then
c ...   Set output file parameters
        req_samples = nboxes
        req_lines = max_samp_line

        fil_lun = out_handle
        fil_datatype = 4
        fil_interleave = 'bil'
        fil_resolution = 5
        fil_desc = 'Direct Broadcast Cloud Top Properties and Cloud '//
     &              'Phase Product'
        fil_error = -327.68
        fil_offset = 0
        fil_samples = nboxes
        fil_lines = nscans * max_samp_line
        fil_bands = OUT_BAND
        out_maxsamples = max_samp_pixel
        out_maxlines = max_samp_line
        init = .false.
      endif

c ... Fill in band and band unit information
      ii = ii + 1
      if (ii .le. OUT_BAND) then
        fil_bandnames(ii) = arrnm
        fil_bandunits(ii) = b_unit_name
      endif

c ... If we have gathered all the band names, then write header
      if (ii .eq. OUT_BAND) then
          hdr_flg = 1
      else
          hdr_flg = 0
      endif

      out = 1
c ... initialize the validity flag holder
      do j = 1 , max_samp_line
        do i = 1 , nboxes
           out_flag(out) = 0
        enddo
        out = out + 1
      enddo

c ... Get the fill value
      
      fill_float = -327.68
      
c ... Create the scaled output array

      out = 1
      
      do j = 1, max_samp_line

        do i = 1, nboxes

c ...     Set default output value (fill value)
        
          float_data( out ) = fill_float

c ...     If array value is good, scale it and store it
        
          if ( abs( array( i, j ) - bad_value ) .gt. 1.0e-5 ) then

            float_data( out ) =  array( i, j ) 

          endif
          
          out = out + 1

        end do

      end do

c ... Write the output array and header to the output files

c ------ write the mask array to a flat output file
      rtn = db_write_flat_file(
     &                         scan,
     &                         req_samples,
     &                         req_lines,
     &                         req_band,
     &                         fil_lun,
     &                         hdr_flg,
     &                         hdr_lun,
     &                         fil_desc,
     &                         fil_datatype,
     &                         fil_interleave,
     &                         fil_resolution,
     &                         fil_error,
     &                         fil_offset,
     &                         fil_samples,
     &                         fil_lines,
     &                         fil_bands,
     &                         fil_bandnames,
     &                         fil_bandunits,
     &                         out_maxsamples,
     &                         out_maxlines,
     &                         float_data,
     &                         out_flag,
     &                         out_samples,
     &                         out_lines
     &                       )

      if ( rtn .ne. 0 ) then
        write(h_output, '(''Write failed on output array '',a)') arrnm
        write(h_output, '(5x,''rtn status = '',I9)') rtn
        write(text, '(''Write failed on output array '',a)') arrnm
        call message( 'db_mod06_write_products', text ,
     &   0, 2 )
      endif
                
      end

c-----------------------------------------------------------------------

      subroutine write_qa(qa_handle, qa_hdr_lun, scan, nscans, nboxes )

c-----------------------------------------------------------------------
c
c!F77
c
c!Description:
c    Write MOD06 QA data to the output file.
c
c!Input Parameters:
c    QA_HANDLE     LUN for MOD06CT QA product
c    QA_HDR_LUN    LUN for QA product header output file
c    SCAN          Scan number within L1B granule
c    NSCANS        Total number of scans in this data set
c    NBOXES        Number of retrieval boxes along the scan
c                  (e.g. 1354 pixels, 5x5 sampling gives 1354/5 boxes)
c
c!Output Parameters:
c    None
c
c    The following arrays in COMMON /MOD06_DATA/ are written to output:
c    PRODUCT_QA            Product run time QA
c
c!Revision History:
c    12-DEC-1997 Liam Gumley, CIMSS/SSEC
c                Created
c
c!Team-unique Header:
c
c!End
c
c-----------------------------------------------------------------------


      implicit none
      save


      include 'db_mod06uw_data.inc'
      include 'db_mod06uw_debug.inc'

c ... arguments

      integer qa_handle, qa_hdr_lun, scan, nboxes, nscans

c ... local variables

      logical init
      byte fil_error
      integer out, i, j, k, rtn, out_samples, out_lines, fil_datatype,
     &        fil_lun, fil_resolution, fil_offset, fil_samples,
     &        fil_lines, fil_bands, out_maxsamples, out_maxlines,
     &        req_lines, req_samples, hdr_flg

c ... local arrays
      byte byte_data( MAX_SAMP_PIXEL * MAX_SAMP_LINE * NPRODUCT_QA ),
     &     out_flag( MAX_SAMP_PIXEL * MAX_SAMP_LINE * NPRODUCT_QA )

      character*4 fil_interleave
      character*72 text
      character*255 fil_bandnames, fil_bandunits, fil_desc

c ... external routines
      external db_write_bsq_file
      integer db_write_bsq_file

      data init /.true./


      hdr_flg = 0
      if (init) then
c ...   Set output file parameters
        req_samples = nboxes
        req_lines = max_samp_line

        fil_lun = qa_handle
        fil_datatype = 1
        fil_interleave = 'bsq'
        fil_resolution = 5
        fil_desc = 'Direct Broadcast Cloud Top Properties and Cloud '//
     &              'Phase Quality Assurance Product'
        fil_error = -1
        fil_offset = 0
        fil_samples = nboxes
        fil_lines = nscans * MAX_SAMP_LINE
        fil_bands = NPRODUCT_QA
        fil_bandnames = 'Byte 1, Byte 2, Byte 3, Byte 4, Byte 5, '//
     &                  'Byte 6, Byte 7, Byte 8, Byte 9, Byte 10'
        fil_bandunits = 'flg, flg, flg, flg, flg, '//
     &                  'flg, flg, flg, flg, flg'
        out_maxsamples = MAX_SAMP_PIXEL
        out_maxlines = MAX_SAMP_LINE 
      endif

c ... Create the product QA output array

      do i = 1, MAX_SAMP_PIXEL * MAX_SAMP_LINE * NPRODUCT_QA
        byte_data( i ) = 0
      end do

      out = 1
c ..  Not the most efficient way to do this .. the original array
c ... is not in the order we would probably want it. 
      do i =  1 , NPRODUCT_QA
        do j = 1 , MAX_SAMP_LINE
           do k = 1 , nboxes
              byte_data( out ) = product_qa( i , k , j ) 
              out_flag( out ) = 0
              out = out + 1 
           enddo
        enddo
      enddo
     
      if (init) then
          hdr_flg = 1
          init = .false.
      else 
          hdr_flg = 0
      endif

c ... Write the product QA array to the output file
c ...  this one is byte sequential
      rtn = db_write_bsq_file(
     &                         scan,
     &                         req_samples,
     &                         req_lines,
     &                         fil_lun,
     &                         hdr_flg,
     &                         qa_hdr_lun,
     &                         fil_desc,
     &                         fil_datatype,
     &                         fil_interleave,
     &                         fil_resolution,
     &                         fil_error,
     &                         fil_offset,
     &                         fil_samples,
     &                         fil_lines,
     &                         fil_bands,
     &                         fil_bandnames,
     &                         fil_bandunits,
     &                         out_maxsamples,
     &                         out_maxlines,
     &                         byte_data,
     &                         out_flag,
     &                         out_samples,
     &                         out_lines
     &                       )

      if ( rtn .ne. 0 ) then
        write(h_output, '(5x,''rtn status = '',I9)') rtn
        write(text, '(''Write failed on output array MOD06QC'')') 
        call message( 'db_mod06_write_products', text ,
     &   0, 2 )
      endif

      end

c-----------------------------------------------------------------------


