C--------------------------------------------------------------------
C  Copyright (C) 2002,  Space Science and Engineering Center, 
C  University C  of Wisconsin-Madison, Madison WI.
C      
C  This program is free software; you can redistribute it 
C  and/or modify it under the terms of the GNU General 
C  Public License as published by the Free Software Foundation; 
C  either version 2 of the License, or (at your option) any 
C  later version.
C
C  This program is distributed in the hope that it will be 
C  useful, but WITHOUT ANY WARRANTY; without even the implied 
C  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
C  See the  GNU General Public License for more details.
C
C  You should have received a copy of the GNU General Public 
C  License along with this program; if not, write to the Free 
C  Software Foundation, Inc., 59 Temple Place, Suite 330, 
C  Boston, MA  02111-1307 USA
C--------------------------------------------------------------------
C
C
      subroutine db_mod06uw_stats_out(ng_co2,ng_irp,
     +                     nmpix, nl, nm, nh, nn, nk, np, nc,
     +                     ni, nw, nx, nu, success_co2, success_irp,
     +                     plow, pmid, phigh, pthin, pthick, popq, 
     +                     pcirrus, pice, pwater, pmixed, punc)


      implicit none
      save

c----------------------------------------------------------------------
C!F77 
c
c!Description:
c     Routine for calculating granule level output statistics
c
c!Input parameters:
c ng_co2        Number of good co2 retrievals
c ng_irp        Number of good IRPHASE retrievals
c nmpix         Total number of pixels used to produce stats
c nl            Number of low cloud pixels found
c nm            Number of mid cloud pixels found
c nh            Number of high cloud pixels found
c nn            Number of thin cloud pixels found
c nk            Number of thick cloud pixels found
c np            Number of opaque cloud pixels found
c nc            Number of cirrus cloud pixels found
c ni            Number of pixels with ice cloud found
c nw            Number of pixels with water cloud found
c nx            Number of pixels with mixed cloud found
c nu            Number of pixels with uncertain clouds found
c
c!Output Parameters: None
c
c success_co2   Percentage of pixels with successful CO2 retrievals
c success_irp   Percentage of pixels with successful irp retrievals
c plow          Percentage of pixels found to have low cloud
c pmid          Percentage of pixels found to have mid cloud
c phigh         Percentage of pixels found to have high cloud
c pthin         Percentage of pixels found to have thin cloud
c pthick        Percentage of pixels found to have thick cloud
c popq          Percentage of pixels found to have opaque cloud
c pcirrus       Percentage of pixels found to have cirrus cloud
c pice          Percentage of pixels found to have ice cloud
c pwater        Percentage of pixels found to have water cloud
c pmixed        Percentage of pixels found to have mixed phase cloud
c punc          Percentage of pixels found to have uncertain phase cloud
c
c!Revision History:
c
c!Team-Unique Header:
c
c!References and Credits:
c See Cloud Mask ATBD-MOD-06.
c
c!END
c----------------------------------------------------------------------
      include 'db_mod06uw_debug.inc'
      include 'db_mod06uw_data.inc'

c     scalar arguments
      integer ng_co2, ng_irp, nmpix, nl, nm, nh, nn, nk, np, nc,
     +        ni, nw, nx, nu
      real success_co2, success_irp,
     +     plow, pmid, phigh, pthin, pthick, popq,
     +     pcirrus, pice, pwater, pmixed, punc


c ... debug statement ............................................
      if (debug .gt. 0) then
        write(h_output,'(10x/,''Within stats_out routine '',/)')
      endif
c ...............................................................

      if(nmpix .gt. 0) then
         success_co2 = (float(ng_co2) / nmpix) * 100.0
         success_irp = (float(ng_irp) / nmpix) * 100.0
         plow   = (float(nl) / nmpix) * 100.0
         pmid   = (float(nm) / nmpix) * 100.0
         phigh  = (float(nh) / nmpix) * 100.0
         pthin  = (float(nn) / nmpix) * 100.0
         pthick = (float(nk) / nmpix) * 100.0
         popq   = (float(np) / nmpix) * 100.0
         pcirrus= (float(nc) / nmpix) * 100.0
         pice   = (float(ni) / nmpix) * 100.0
         pwater = (float(nw) / nmpix) * 100.0
         pmixed = (float(nx) / nmpix) * 100.0
         punc   = (float(nu) / nmpix) * 100.0
      else
c        write error message to log file
         call message( 'stats_out',
     &   'Warning - No good pixels were found to process ',
     &   0, 3 )

         success_co2 = 0.0
         success_irp = 0.0
         plow   =  META_MISG
         pmid   =  META_MISG
         phigh  =  META_MISG
         pthin  =  META_MISG
         pthick =  META_MISG
         popq   =  META_MISG
         pcirrus=  META_MISG
         pice   =  META_MISG
         pwater =  META_MISG
         pmixed =  META_MISG
         punc   =  META_MISG
      end if

      write (h_output,fmt='(/,15x,''MODIS DB CTP Granule Statstics: '',
     +        /)')
      write (h_output,fmt='(5x,''Parameter         Total Number '',
     +       ''    Percent '',/)')
      write (h_output,fmt='(5x,''Good CO2 rets. '',I10,f15.2)')
     +       ng_co2,success_co2
      write (h_output,fmt='(5x,''Good IRP rets. '',I10,f15.2)')
     +       ng_irp,success_irp
      write (h_output,fmt='(5x,''Low Cloud      '',I10,f15.2)')
     +       nl,plow
      write (h_output,fmt='(5x,''Mid Cloud      '',I10,f15.2)')
     +       nm,pmid
      write (h_output,fmt='(5x,''High Cloud     '',I10,f15.2)')
     +       nh,phigh
      write (h_output,fmt='(5x,''Thin Cloud     '',I10,f15.2)')
     +       nn,pthin
      write (h_output,fmt='(5x,''Thick Cloud    '',I10,f15.2)')
     +       nk,pthick
      write (h_output,fmt='(5x,''Cirrus Cloud   '',I10,f15.2)')
     +       nc,pcirrus
      write (h_output,fmt='(5x,''Opaque Cloud   '',I10,f15.2)')
     +       np,popq
      write (h_output,fmt='(5x,''Ice Cloud      '',I10,f15.2)')
     +       ni,pice
      write (h_output,fmt='(5x,''Water Cloud    '',I10,f15.2)')
     +       nw,pwater
      write (h_output,fmt='(5x,''Mixed Phase    '',I10,f15.2)')
     +       nx,pmixed
      write (h_output,fmt='(5x,''Uncertain Phase'',I10,f15.2,/)')
     +       nu,punc
      write (h_output,fmt='(4x)')
      write (h_output,fmt='(1x,''Total pixels: '',i10)')nmpix

      return
      end
